-------------------------------------------------------------------------------
--|> Player Bars
--: Manages the display of the bars and their elements representing detected players
-------------------------------------------------------------------------------
local _, NS = ...
local UIFrameFadeIn = UIFrameFadeIn
local UIFrameFadeOut = UIFrameFadeOut
local InCombatLockdown = InCombatLockdown
local GetTime = GetTime

--> Core UI
local CUI = NS.CoreUI

--|> LIBS
-------------------------------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

--> Build Bars
local Bar = {}
function CUI.BuildBars()
    CUI.Bar = CUI.Bar or {}
    Bar = CUI.Bar
    for k = 1, NS.Options.Bars.MaxNumBars do
        Bar[k] = Bar[k] or CreateFrame("StatusBar", "Bar" .. k, weizPVP_CoreFrame.ScrollFrame.ListFrame)
        Bar[k].id = k
        Bar[k]:SetHeight(NS.Options.Bars.RowHeight)
        --: Index Offset
        if k == 1 then
            Bar[k]:SetPoint("TOP", weizPVP_CoreFrame.ScrollFrame.ListFrame, "TOP")
        else
            Bar[k]:SetPoint("TOP", Bar[(k - 1)], "BOTTOM", 0, (-1 * NS.Options.Bars.VerticalSpacing))
        end
        Bar[k]:SetPoint("RIGHT", weizPVP_CoreFrame.ScrollFrame, "RIGHT", -1, 0)
        Bar[k]:SetPoint("LEFT", weizPVP_CoreFrame.ScrollFrame, "LEFT", 1, 0)
        Bar[k]:SetStatusBarTexture(SM:Fetch("statusbar", NS.Options.Bars.Texture))
        Bar[k]:SetStatusBarColor(0, 0, 0, 0)
        Bar[k]:SetMinMaxValues(0, 100)
        Bar[k]:EnableMouse(true)
        Bar[k]:SetValue(100)
        --: bg
        Bar[k].bg = Bar[k].bg or Bar[k]:CreateTexture(nil, "BACKGROUND")
        Bar[k].bg:SetTexture(SM:Fetch("background", NS.Options.Bars.BarTexture))
        Bar[k].bg:SetAllPoints(true)
        Bar[k].bg:SetVertexColor(0, 0, 0, 0)
        --: Highlight
        Bar[k].Highlight = Bar[k].Highlight or Bar[k]:CreateTexture(nil, "ARTWORK", nil, 1)
        Bar[k].Highlight:SetAllPoints()
        Bar[k].Highlight:SetColorTexture(1, 1, 1)
        Bar[k].Highlight:SetBlendMode("ADD")
        Bar[k].Highlight:SetAlpha(0)
        --: KOS
        Bar[k].KOSRibbon = Bar[k].KOSRibbon or Bar[k]:CreateTexture(nil, "ARTWORK", nil, 5)
        Bar[k].KOSRibbon:SetTexture("Interface\\Addons\\weizPVP\\Addons\\KOS\\Media\\kos_ribbon.tga")
        Bar[k].KOSRibbon:SetPoint("LEFT", Bar[k], "LEFT")
        Bar[k].KOSRibbon:SetPoint("TOP", Bar[k], "TOP")
        Bar[k].KOSRibbon:SetPoint("BOTTOM", Bar[k], "BOTTOM")
        Bar[k].KOSRibbon:SetWidth(NS.Options.Bars.RowHeight)
        --: ROLE ICON
        Bar[k].RoleIcon = Bar[k].RoleIcon or Bar[k]:CreateTexture(nil, "ARTWORK", nil, 2)
        Bar[k].RoleIcon:SetPoint("LEFT", Bar[k], "LEFT", 2, 0)
        Bar[k].RoleIcon:SetSize(NS.Options.Bars.RowHeight - 3, NS.Options.Bars.RowHeight - 3)
        -- Bar[k].RoleIcon:SetTexture("Interface\\Addons\\weizPVP\\Media\\Icons\\unknown.tga", false)
        --: Name
        Bar[k].Name = Bar[k].Name or Bar[k]:CreateFontString(nil, "ARTWORK", nil, 2)
        Bar[k].Name:SetFont(SM:Fetch("font", NS.Options.Bars.NameFont), NS.Options.Bars.NameFontSize, "OUTLINE")
        Bar[k].Name:SetHeight(NS.Options.Bars.NameFontSize)
        Bar[k].Name:SetPoint("LEFT", Bar[k].RoleIcon, "RIGHT", 2, 0)
        Bar[k].Name:SetDrawLayer("OVERLAY", 7)
        Bar[k].Name:SetJustifyH("LEFT")
        Bar[k].Name:SetJustifyV("MIDDLE")
        Bar[k].Name:SetTextColor(1, 1, 1, 1)
        Bar[k].Name:SetShadowColor(0.2, 0.2, 0.2, 0.4)
        Bar[k].Name:SetShadowOffset(1, -1)
        -- Bar[k].Name:SetText("Name" .. tostring(k))
        --: Level
        Bar[k].Level = Bar[k].Level or Bar[k]:CreateFontString(nil, "ARTWORK", GameFontNormal, 2)
        Bar[k].Level:SetFont(SM:Fetch("font", NS.Options.Bars.LevelFont), NS.Options.Bars.LevelFontSize, "OUTLINE")
        Bar[k].Level:SetHeight(NS.Options.Bars.LevelFontSize)
        Bar[k].Level:SetPoint("LEFT", Bar[k].Name, "RIGHT", 2, 0)
        Bar[k].Level:SetJustifyH("LEFT")
        Bar[k].Level:SetTextColor(1, 1, 1, 1)
        Bar[k].Level:SetShadowColor(0, 0, 0, 0)
        -- Bar[k].Level:SetText("Level")
        --: Guild
        Bar[k].Guild = Bar[k].Guild or Bar[k]:CreateFontString(nil, "ARTWORK", GameFontNormal, 2)
        Bar[k].Guild:SetFont(SM:Fetch("font", NS.Options.Bars.GuildFont), NS.Options.Bars.GuildFontSize, "OUTLINE")
        Bar[k].Guild:SetHeight(NS.Options.Bars.GuildFontSize)
        Bar[k].Guild:SetMaxLines(1)
        Bar[k].Guild:SetPoint("RIGHT", Bar[k], "RIGHT", -4, 0)
        Bar[k].Guild:SetJustifyH("RIGHT")
        Bar[k].Guild:SetJustifyV("MIDDLE")
        Bar[k].Guild:SetTextColor(1, 1, 1, 1)
        Bar[k].Guild:SetShadowColor(0, 0, 0, 0)
        -- Bar[k].Guild:SetText("Guild")
        --: DEAD ICON
        Bar[k].DeadIcon = Bar[k].DeadIcon or Bar[k]:CreateTexture(nil, "ARTWORK", nil, 2)
        Bar[k].DeadIcon:SetPoint("LEFT", Bar[k].Level, "RIGHT", 4, 0)
        Bar[k].DeadIcon:SetSize(NS.Options.Bars.RowHeight, NS.Options.Bars.RowHeight)
        Bar[k].DeadIcon:SetTexture("Interface\\Addons\\weizPVP\\Media\\Icons\\dead.tga", false)
        Bar[k].DeadIcon:Hide()
        --> Button
        Bar[k].Button = Bar[k].Button or CreateFrame("Button", nil, Bar[k], "InsecureActionButtonTemplate")
        Bar[k].Button:SetPoint("TOPRIGHT")
        Bar[k].Button:SetPoint("TOPLEFT")
        Bar[k].Button:SetHeight(Bar[k]:GetHeight() + NS.Options.Bars.VerticalSpacing)
        Bar[k].Button:SetAlpha(0)
        Bar[k].Button.id = k
        Bar[k].Button:SetScript(
            "OnLoad",
            function(self)
                if not InCombatLockdown() then
                    Bar[k].Button:SetAttribute("type1", "macro")
                    Bar[k].Button:SetAttribute("macrotext1", "/targetexact " .. self.Target)
                end
            end
        )
        Bar[k].Button:SetScript(
            "PostClick",
            function(_, button)
                CUI.ButtonPostClick(k, button)
            end
        )
        Bar[k].Button:SetScript(
            "OnMouseDown",
            function(_, button)
                CUI.ButtonPreClick(k, button)
            end
        )
        Bar[k].Button:SetScript(
            "OnEnter",
            function()
                UIFrameFadeIn(Bar[k].Highlight, 0.05, 0, 0.2)
            end
        )
        Bar[k].Button:SetScript(
            "OnLeave",
            function()
                UIFrameFadeOut(Bar[k].Highlight, 0.2, 0.2, 0)
            end
        )
    end
end

--|> SECURE BUTTON ACTIONS
-------------------------------------------------------------------------------
--> PLAYER BAR: PRE-CLICK
function CUI.ButtonPreClick(barID, MouseButton)
    if not Bar[barID].name then
        return
    end
    local displayName = Bar[barID].displayName
    local playerName = Bar[barID].name
    UIFrameFadeIn(Bar[barID].Highlight, 0.05, 0.15, 0.3)
    if playerName then
        if MouseButton == "LeftButton" then
            if not InCombatLockdown() then
                Bar[barID].Button:SetAttribute("type1", "macro")
                Bar[barID].Button:SetAttribute("macrotext", "/targetexact " .. displayName)
                Bar[barID].Button.Target = displayName
            end
        elseif MouseButton == "RightButton" then
            NS.PlayerBarMenu_OnClick(Bar[barID])
        end
    end
end

--> PLAYER BAR: POST-CLICK
function CUI.ButtonPostClick(barID, MouseButton)
    local playerName = Bar[barID].displayName
    local targetSetName = Bar[barID].Button.Target
    local playerGUID = Bar[barID].GUID
    local targetName = GetUnitName("target", true)
    if playerName and NS.PlayerActiveCache[playerGUID] then
        if MouseButton == "LeftButton" and targetName ~= playerName then
            if not NS.PlayerActiveCache[playerGUID].Stealth then
                if targetSetName == playerName then
                    if NS.ActiveList[playerGUID] then
                        NS.InactiveList[playerGUID] = NS.ActiveList[playerGUID]
                        NS.InactiveList[playerGUID].TimeUpdated = GetTime() + NS.Options.Sorting.NearbyActiveTimeout + 0.1
                        NS.ActiveList[playerGUID] = nil
                    elseif NS.ActiveDeadList[playerGUID] then
                        NS.InactiveDeadList[playerGUID] = NS.ActiveDeadList[playerGUID]
                        NS.InactiveDeadList[playerGUID].TimeUpdated = GetTime() + NS.Options.Sorting.NearbyActiveTimeout + 0.1
                        NS.ActiveDeadList[playerGUID] = nil
                    end
                    NS.SortNearbyList()
                    NS.RefreshCurrentList()
                end
            end
        end
    end
end
