-----------------------------------------------------------
--|> UNIT PROCESSING
--: Process Unit Data
-----------------------------------------------------------
local _, NS = ... --> Addon Name, Namespace
--> UpValues
local UnitName = UnitName
local GetGuildInfo = GetGuildInfo
local GetTime = GetTime
local string_find = string.find
local UnitCanAttack = UnitCanAttack
local UnitExists = UnitExists
local UnitClass = UnitClass
local UnitRace = UnitRace
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local UnitOnTaxi = UnitOnTaxi
local UnitIsPlayer = UnitIsPlayer
local GetUnitName = GetUnitName
local UnitFactionGroup = UnitFactionGroup
local UnitLevel = UnitLevel
local gsub = gsub
local select = select

--> GET UNIT DATA
local unitUpdateThreshold = 120 -- 2 minutes between guild/race/level/role checks
-- Gets data for events with a UNIT ID; such as 'target' or 'nameplate'
function NS.GetUnitData(unit)
    if (not unit) or (not UnitExists(unit)) or (not NS.IsUnitValidForTracking(unit)) then
        return -- no valid unit to process
    end
    -- valid unit exists: get info!
    local name = NS.GetUnitName(unit)
    local GUID = UnitGUID(unit) or nil
    if name and GUID then
        --: Add player to DB if not found
        if not NS.PlayerDB[name] then
            NS.PlayerDB[name] = {}
            NS.PlayerDB[name].Name = name
            NS.PlayerDB[name].Timestamp = 0
            _, NS.PlayerDB[name].Class = UnitClass(unit)
            NS.PlayerDB[name].Role = NS.ClassRoleAssign(NS.PlayerDB[name].Class)
        end

        --: Update player info if estimated or past update threshold
        if NS.PlayerDB[name].Timestamp + unitUpdateThreshold > GetTime() or NS.PlayerDB[name].Estimated == true then
            NS.PlayerDB[name].Guild = GetGuildInfo(unit)
            NS.PlayerDB[name].Level = UnitLevel(unit)
            NS.PlayerDB[name].Race = UnitRace(unit)
        end
        --: Player On Bars?
        if NS.PlayersOnBars[GUID] then
            NS.PlayerActiveCache[GUID].OnTaxi = UnitOnTaxi(unit)
            if select(1, UnitFactionGroup(unit)) == NS.Player.Faction then
                NS.PlayerActiveCache[GUID].SameFaction = true
            end
            NS.UnitHealthCheck(unit)
        end
        NS.PlayerDB[name].Estimated = false
        NS.PlayerDB[name].Timestamp = GetTime()
        NS.UpdatePlayerActiveCache(name, nil, nil, nil, GUID) -- (name, stealth, dead, role, GUID)
        return
    end
end

--> Is Unit Valid For Tracking
function NS.IsUnitValidForTracking(unit)
    if (not unit) or (not UnitIsPlayer(unit)) then -- input check
        return false
    end
    if string_find(GetUnitName(unit), "Unknown") then
        return false
    end
    if (UnitCanAttack("player", unit) or UnitIsEnemy("player", unit) or NS.Options.DEBUG) then -- enemy player check
        return true
    else
        if not NS.Options.DEBUG and NS.PlayerActiveCache[UnitGUID(unit)] then
            local GUID = UnitGUID(unit) or nil
            NS.PlayerActiveCache[GUID] = nil
            if NS.ActiveList[GUID] then
                NS.ActiveList[GUID].TimeAdded = 0
                NS.ActiveList[GUID].TimeUpdated = 0
            end
            if NS.ActiveDeadList[GUID] then
                NS.ActiveDeadList[GUID].TimeAdded = 0
                NS.ActiveDeadList[GUID].TimeUpdated = 0
            end
            if NS.InactiveList[GUID] then
                NS.InactiveList[GUID].TimeAdded = 0
                NS.InactiveList[GUID].TimeUpdated = 0
            end
            if NS.InactiveDeadList[GUID] then
                NS.InactiveDeadList[GUID].TimeAdded = 0
                NS.InactiveDeadList[GUID].TimeUpdated = 0
            end
            NS.CurrentNameplates[GUID] = nil
            if NS.NearbyCount == 1 then
              NS.CoreUI.Initialize()
            end
            NS.ManageListTimeouts()
        end
    end
    return false
end

--> weizPVP: GetUnitName()
--: @returns "playerName-realmName" for all players, no matter the realm
function NS.GetUnitName(unit)
    if not unit then -- check for unit
        return
    end
    local name, realm = UnitName(unit, true) -- check for unit name
    if not name then
        return
    end
    if not realm then -- same realm
        return name .. "-" .. NS.PlayerRealm
    else -- different realm
        realm = gsub(realm, "[%s%-]", "")
        return name .. "-" .. realm
    end
end

--> UnitHealthCheck
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local UnitGUID = UnitGUID
local GUID
function NS.UnitHealthCheck(unit)
    GUID = UnitGUID(unit) or nil
    if GUID and UnitExists(unit) then
        NS.PlayerActiveCache[GUID].Unattackable = true and not UnitCanAttack("player", unit) or nil
        NS.PlayerActiveCache[GUID].Dead = UnitIsDeadOrGhost(unit)
        NS.PlayerActiveCache[GUID].Health = UnitHealth(unit) / UnitHealthMax(unit) * 100
        NS.RefreshBarByGUID(GUID)
    end
end
